#ifndef MPC555_WALLCLOCK_CXX
#define MPC555_WALLCLOCK_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "pkgconf/wallclock.h"

#include "cyg/infra/cyg_type.h"
#include "cyg/devs/wallclock.hxx"
#include "cyg/hal/hal_io.h"
#include "cyg/hal/mpc555_regs.h"

#include "cyg/kernel/sched.hxx"
#include "cyg/kernel/sched.inl"

#ifndef CYGIMP_WALLCLOCK_EMULATE // Make sure dubbel definitions are avoided. "emulate.cxx" is
                                 // allways included in the build
//---------------------
// Class implementation
//---------------------
// MPC555 Hardware implements a real time clock. No need to keep track of seconds in software
// Get current time only has to read the RTC register. Set current time needs nothing to do but
// write to the RTC register. There really is no need to use interrupts for doing this.

//--------------------------------------
// Allocate memory for static attributes
//--------------------------------------
static Cyg_WallClock wallclock_instance CYG_INIT_PRIORITY( CLOCK );
Cyg_WallClock * Cyg_WallClock::wallclock; 

// Public constructor(s) and destructor
Cyg_WallClock::Cyg_WallClock()
{
  // Install the instance pointer
  wallclock = &wallclock_instance;
}

//---------------------
// Other public members
//---------------------
// get_current_time
cyg_uint32 Cyg_WallClock::get_current_time(void)
{
  cyg_uint32 result;
  
  Cyg_Scheduler::lock();
  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_RTC);    // Keep this register locked, so no-one but the
  HAL_READ_UINT32(CYGARC_REG_IMM_RTC, result);  // wallclock can mess with time
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_RTC);
  Cyg_Scheduler::unlock();

  return result;
}

// set_current_time
void Cyg_WallClock::set_current_time( cyg_uint32 time_stamp)
{
  Cyg_Scheduler::lock();
  CYGARC_UNLOCKREGISTER(CYGARC_REG_IMM_RTC);
  HAL_WRITE_UINT32(CYGARC_REG_IMM_RTC, time_stamp);
  CYGARC_LOCKREGISTER(CYGARC_REG_IMM_RTC);
  Cyg_Scheduler::unlock();
}
#endif // CYGIMP_WALLCLOCK_EMULATE

#endif // MPC555_WALLCLOCK_CXX
