#ifndef CYGONCE_HAL_IMMR_H
#define CYGONCE_HAL_IMMR_H

#if defined(__GNUC__) || defined(__GNUG__)
// Leave these definitions out for assembler includes
// General register definitions
extern unsigned long  SIUMCR;
extern unsigned long  SYPCR;
extern unsigned short SWSR;
extern unsigned long  SIPEND;
extern unsigned long  SIMASK;
extern unsigned long  SIEL;
extern unsigned long  SIVEC;
extern unsigned long  TESR;
extern unsigned long  SGPIODT1;
extern unsigned long  SGPIODT2;
extern unsigned long  SGPIOCR;
extern unsigned long  EMCR;
extern unsigned long  PDMCR;

// Memory controller registers
extern unsigned long  BR0;
extern unsigned long  OR0;
extern unsigned long  BR1;
extern unsigned long  OR1;
extern unsigned long  BR2;
extern unsigned long  OR2;
extern unsigned long  BR3;
extern unsigned long  OR3;
extern unsigned long  DMBR;
extern unsigned long  DMOR;
extern unsigned short MSTAT;

// System integration timers
extern unsigned short TBSCR;
extern unsigned long  TBREF0;
extern unsigned long  TBREF1;
extern unsigned short RTCSC;
extern unsigned long  RTC;
extern unsigned long  RTSEC;
extern unsigned long  RTCAL;
extern unsigned short PISCR;
extern unsigned long  PITC;
extern unsigned long  PITR;

// Clocks and resets
extern unsigned long  SCCR;
extern unsigned long  PLPRCR;
extern unsigned short RSR;
extern unsigned short COLIR;
extern unsigned short VSRMCR;

// System IntegrationTimer Keys
extern unsigned long  TBSCRK;
extern unsigned long  TBREF0K;
extern unsigned long  TBREF1K;
extern unsigned long  TBK;
extern unsigned long  RTCSCK;
extern unsigned long  RTCK;
extern unsigned long  RTSECK;
extern unsigned long  RTCALK;
extern unsigned long  PISCRK;
extern unsigned long  PITCK;

// Clocks and reset keys
extern unsigned long  SCCRK;
extern unsigned long  PLPRCRK;
extern unsigned long  RSRK;
 
// -------------------------------------
//  CMF (CDR Mone T FLASH EEPROM)
// -------------------------------------
// CMF_A
extern unsigned long  CMFMCR_A;
extern unsigned long  CMFTST_A;
extern unsigned long  CMFCTL_A;
	
// CMF_B
extern unsigned long  CMFMCR_B;
extern unsigned long  CMFTST_B;
extern unsigned long  CMFCTL_B;

//-------------------------------------
// DPTRAM (Dual-Port TPU RAM)
//-------------------------------------
extern unsigned short DPTMCR;
extern unsigned short RAMTST;
extern unsigned short RAMBAR;
extern unsigned short MISRH;
extern unsigned short MISRL;
extern unsigned short MISCNT;

//-------------------------------------
// TPU3 (Time processing unit)
//-------------------------------------
// TPU-A
extern unsigned short TPUMCR_A;
extern unsigned short TCR_A;
extern unsigned short DSCR_A;
extern unsigned short DSSR_A;
extern unsigned short TICR_A;
extern unsigned short CIER_A;
extern unsigned short CFSR0_A;
extern unsigned short CFSR1_A;
extern unsigned short CFSR2_A;
extern unsigned short CFSR3_A;
extern unsigned short HSQR0_A;
extern unsigned short HSQR1_A;
extern unsigned short HSRR0_A;
extern unsigned short HSRR1_A;
extern unsigned short CPR0_A;
extern unsigned short CPR1_A;
extern unsigned short CISR_A;
extern unsigned short LR_A;
extern unsigned short SGLR_A;
extern unsigned short DCNR_A;
extern unsigned short TPUMCR2_A;
extern unsigned short TPUMCR3_A;
extern unsigned short ISDR_A;
extern unsigned short ISCR_A;

// TPU-B
extern unsigned short TPUMCR_B;
extern unsigned short TCR_B;
extern unsigned short DSCR_B;
extern unsigned short DSSR_B;
extern unsigned short TICR_B;
extern unsigned short CIER_B;
extern unsigned short CFSR0_B;
extern unsigned short CFSR1_B;
extern unsigned short CFSR2_B;
extern unsigned short CFSR3_B;
extern unsigned short HSQR0_B;
extern unsigned short HSQR1_B;
extern unsigned short HSRR0_B;
extern unsigned short HSRR1_B;
extern unsigned short CPR0_B;
extern unsigned short CPR1_B;
extern unsigned short CISR_B;
extern unsigned short LR_B;
extern unsigned short SGLR_B;
extern unsigned short DCNR_B;
extern unsigned short TPUMCR2_B;
extern unsigned short TPUMCR3_B;
extern unsigned short ISDR_B;
extern unsigned short ISCR_B;

//-------------------------------------
// QADC64 (Queued Analog-to-digital Converter)
//-------------------------------------
// QUADC-A
extern unsigned short QUADC64MCR_A;
extern unsigned short QUADC64TEST_A;
extern unsigned short QUADC64INT_A;
extern unsigned short PORTQA_A_PORTQB_A;
extern unsigned short DDRQA_A_DDRQB_A;
extern unsigned short QUACR0_A;
extern unsigned short QUACR1_A;
extern unsigned short QUACR2_A;
extern unsigned short QUASR0_A;
extern unsigned short QUASR1_A;
extern unsigned short CCW_A;
extern unsigned short RJURR_A;
extern unsigned short LJSRR_A;
extern unsigned short LJURR_A;

// QUADC-B
extern unsigned short QUADC64MCR_B;
extern unsigned short QUADC64TEST_B;
extern unsigned short QUADC64INT_B;
extern unsigned short PORTQA_B_PORTQB_B;
extern unsigned short DDRQA_B_DDRQB_B;
extern unsigned short QUACR0_B;
extern unsigned short QUACR1_B;
extern unsigned short QUACR2_B;
extern unsigned short QUASR0_B;
extern unsigned short QUASR1_B;
extern unsigned short CCW_B;
extern unsigned short RJURR_B;
extern unsigned short LJSRR_B;
extern unsigned short LJURR_B;

//-------------------------------------
// QSMCM (Queued Serial Multi-Channel Module)
//-------------------------------------
extern unsigned short QSMCMMCR;
extern unsigned short QTEST;
extern unsigned short QDSCI_IL;
extern unsigned short QSPI_IL;
extern unsigned short SCC1R0;
extern unsigned short SCC1R1;
extern unsigned short SC1SR;
extern unsigned short SC1DR;
extern unsigned short PORTQS;
extern unsigned short PQSPAR_DDRQST;
extern unsigned short SPCR0;
extern unsigned short SPCR1;
extern unsigned short SPCR2;
extern unsigned char  SPCR3;
extern unsigned char  SPSR;
extern unsigned short SCC2R0;
extern unsigned short SCC2R1;
extern unsigned short SC2SR;
extern unsigned short SC2DR;
extern unsigned short QSCI1CR;
extern unsigned short QSCI1SR;
extern unsigned short SCTQ;
extern unsigned short SCRQ;
extern unsigned short RECRAM;
extern unsigned short TRAN_RAM;
extern unsigned short COMD_RAM;

//-------------------------------------
// MIOS1 (Modular Input/Output Subsystem)
//-------------------------------------
// MPIOS Pulse width modulation submodule 0
extern unsigned short MPWMSMPERR_0;
extern unsigned short MPWMSMPULR_0;
extern unsigned short MPWMSMCNTR_0;
extern unsigned short MPWMSMSCR_0;
		
// MPIOS Pulse width modulation submodule 1
extern unsigned short MPWMSMPERR_1;
extern unsigned short MPWMSMPULR_1;
extern unsigned short MPWMSMCNTR_1;
extern unsigned short MPWMSMSCR_1;

// MPIOS Pulse width modulation submodule 2
extern unsigned short MPWMSMPERR_2;
extern unsigned short MPWMSMPULR_2;
extern unsigned short MPWMSMCNTR_2;
extern unsigned short MPWMSMSCR_2;

// MPIOS Pulse width modulation submodule 3
extern unsigned short MPWMSMPERR_3;
extern unsigned short MPWMSMPULR_3;
extern unsigned short MPWMSMCNTR_3;
extern unsigned short MPWMSMSCR_3;

// MIOS Modulus counter submodule 6
extern unsigned short MMCSMCNT_6;
extern unsigned short MMCSMML_6;
extern unsigned short MMCSMSCRD_6;
extern unsigned short MMCSMSCR_6;

// MIOS Double Action submodule 11
extern unsigned short MDASMAR_11;
extern unsigned short MDASMBR_11;
extern unsigned short MDASMSCRD_11;
extern unsigned short MDASMSCR_11;

// MIOS Double Action submodule 12
extern unsigned short MDASMAR_12;
extern unsigned short MDASMBR_12;
extern unsigned short MDASMSCRD_12;
extern unsigned short MDASMSCR_12;

// MIOS Double Action submodule 13
extern unsigned short MDASMAR_13;
extern unsigned short MDASMBR_13;
extern unsigned short MDASMSCRD_13;
extern unsigned short MDASMSCR_13;

// MIOS Double Action submodule 14
extern unsigned short MDASMAR_14;
extern unsigned short MDASMBR_14;
extern unsigned short MDASMSCRD_14;
extern unsigned short MDASMSCR_14;

// MIOS Double Action submodule 15
extern unsigned short MDASMAR_15;
extern unsigned short MDASMBR_15;
extern unsigned short MDASMSCRD_15;
extern unsigned short MDASMSCR_15;

// MPIOS Pulse width modulation submodule 16
extern unsigned short MPWMSMPERR_16;
extern unsigned short MPWMSMPULR_16;
extern unsigned short MPWMSMCNTR_16;
extern unsigned short MPWMSMSCR_16;

// MPIOS Pulse width modulation submodule 17
extern unsigned short MPWMSMPERR_17;
extern unsigned short MPWMSMPULR_17;
extern unsigned short MPWMSMCNTR_17;
extern unsigned short MPWMSMSCR_17;

// MPIOS Pulse width modulation submodule 18
extern unsigned short MPWMSMPERR_18;
extern unsigned short MPWMSMPULR_18;
extern unsigned short MPWMSMCNTR_18;
extern unsigned short MPWMSMSCR_18;

// MPIOS Pulse width modulation submodule 19
extern unsigned short MPWMSMPERR_19;
extern unsigned short MPWMSMPULR_19;
extern unsigned short MPWMSMCNTR_19;
extern unsigned short MPWMSMSCR_19;

// MIOS Modulus counter submodule 22
extern unsigned short MMCSMCNT_22;
extern unsigned short MMCSMML_22;
extern unsigned short MMCSMSCRD_22;
extern unsigned short MMCSMSCR_22;

// MIOS Double action submodule 27
extern unsigned short MDASMAR_27;
extern unsigned short MDASMBR_27;
extern unsigned short MDASMSCRD_27;
extern unsigned short MDASMSCR_27;

// MIOS Double action submodule 28
extern unsigned short MDASMAR_28;
extern unsigned short MDASMBR_28;
extern unsigned short MDASMSCRD_28;
extern unsigned short MDASMSCR_28;

// MIOS Double action submodule 29
extern unsigned short MDASMAR_29;
extern unsigned short MDASMBR_29;
extern unsigned short MDASMSCRD_29;
extern unsigned short MDASMSCR_29;

// MIOS Double action submodule 30
extern unsigned short MDASMAR_30;
extern unsigned short MDASMBR_29;
extern unsigned short MDASMSCRD_30;
extern unsigned short MDASMSCR_30;

// MIOS Double action submodule 31
extern unsigned short MDASMAR_31;
extern unsigned short MDASMBR_31;
extern unsigned short MDASMSCRD_31;
extern unsigned short MDASMSCR_31;

// MIOS Paralell port I/O submodule
extern unsigned short MPIOSMDR;
extern unsigned short MPIOSMDDR;

// MIOS Bus interface Submodule
extern unsigned short MIOS1TPCR;
extern unsigned short MIOS1VNR;
extern unsigned short MIOS1MCR;

// MIOS Counter / Prescaler submodule
extern unsigned short MCPSMSCR;

// MIOS Interrupt request submodule 0
extern unsigned short MIOS1SR0;
extern unsigned short MIOS1ER0;
extern unsigned short MIOS1RPR0;
extern unsigned short MIOS1LVL0;

// Mios Interrupt request submodule 1
extern unsigned short MIOS1SR1;
extern unsigned short MIOS1ER1;
extern unsigned short MIOS1RPR1;
extern unsigned short MIOS1LVL1;

//-------------------------------------
// TouCAN (CAN 2.0B Controller)
//-------------------------------------
// TouCAN_A
extern unsigned short TCNMCR_A;
extern unsigned short TTR_A;
extern unsigned short CANICR_A;
extern unsigned short CANCTRL0_A_CANCTRL1_A;
extern unsigned short PRESDIV_A_CTRL2_A;
extern unsigned short TIMER_A;
extern unsigned short RXGMASKHI_A;
extern unsigned short RXGMASKLO_A;
extern unsigned short RX14MASKHI_A;
extern unsigned short  RX14MASKLO_A;
extern unsigned short RX15MASKHI_A;
extern unsigned short RX15MASKLO_A;
extern unsigned short ESTAT_A;
extern unsigned short IMASK_A;
extern unsigned short IFLAG_A;
extern unsigned short RXECTR_A_TXECTR_A;

// TouCAN_B
extern unsigned short TCNMCR_B;
extern unsigned short TTR_B;
extern unsigned short CANICR_B;
extern unsigned short CANCTRL0_B_CANCTRL1_B;
extern unsigned short PRESDIV_B_CTRL2_B;
extern unsigned short TIMER_B;
extern unsigned short RXGMASKHI_B;
extern unsigned short RXGMASKLO_B;
extern unsigned short RX14MASKHI_B;
extern unsigned short RX14MASKLO_B;
extern unsigned short RX15MASKHI_B;
extern unsigned short RX15MASKLO_B;
extern unsigned short ESTAT_B;
extern unsigned short IMASK_B;
extern unsigned short IFLAG_B;
extern unsigned short RXECTR_A_TXECTR_B;

//-------------------------------------
// UIMB (U-Bus to IMB3 Bus Interface)
//-------------------------------------
extern unsigned long  UMCR;
extern unsigned long  UTSTCREG;
extern unsigned long  UIPEND;

//-------------------------------------
// SRAM (Static RAM Access memory)
//-------------------------------------
extern unsigned short SRAMMCR_A;
extern unsigned short SRAMTST_A;
extern unsigned short SRAMMCR_B;
extern unsigned short SRAMTST_B;
#endif // #if defined(__GNUC__) || defined(__GNUG__)

#define IMMR_BASE                  0x002fc000

			// General register definitions
#define IMMR_SIUMCR                0x0000
#define IMMR_SYPCR                 0x0004
#define IMMR_SWSR                  0x000e
#define IMMR_SIPEND                0x0010
#define IMMR_SIMASK                0x0014
#define IMMR_SIEL                  0x0018
#define IMMR_SIVEC                 0x001c
#define IMMR_TESR                  0x0020
#define IMMR_SGPIODT1              0x0024
#define IMMR_SGPIODT2              0x0028
#define IMMR_SGPIOCR               0x002c
#define IMMR_EMCR                  0x0030
#define IMMR_PDMCR                 0x003c

			// Memory controller registers
#define IMMR_BR0                   0x0100
#define IMMR_OR0                   0x0104
#define IMMR_BR1                   0x0108
#define IMMR_OR1                   0x010c
#define IMMR_BR2                   0x0110
#define IMMR_OR2                   0x0114
#define IMMR_BR3                   0x0118
#define IMMR_OR3                   0x011c
#define IMMR_DMBR                  0x0140
#define IMMR_DMOR                  0x0144
#define IMMR_MSTAT                 0x0178

			// System integration timers
#define IMMR_TBSCR                 0x0200
#define IMMR_TBREF0                0x0204
#define IMMR_TBREF1                0x0208
#define IMMR_RTCSC                 0x0220
#define IMMR_RTC                   0x0224
#define IMMR_RTSEC                 0x0228
#define IMMR_RTCAL                 0x022c
#define IMMR_PISCR                 0x0240
#define IMMR_PITC                  0x0244
#define IMMR_PITR                  0x0248

			// Clocks and resets
#define IMMR_SCCR                  0x0280
#define IMMR_PLPRCR                0x0284
#define IMMR_RSR                   0x0288
#define IMMR_COLIR                 0x028c
#define IMMR_VSRMCR                0x0290

			// System IntegrationTimer Keys
#define IMMR_TBSCRK                0x0300
#define IMMR_TBREF0K               0x0304
#define IMMR_TBREF1K               0x0308
#define IMMR_TBK                   0x030c
#define IMMR_RTCSCK                0x0320
#define IMMR_RTCK                  0x0324
#define IMMR_RTSECK                0x0328
#define IMMR_RTCALK                0x032c
#define IMMR_PISCRK                0x0340
#define IMMR_PITCK                 0x0344

			// Clocks and reset keys
#define IMMR_SCCRK                 0x0380
#define IMMR_PLPRCRK               0x0384
#define IMMR_RSRK                  0x0388

		//-------------------------------------
		// CMF (CDR Mone T FLASH EEPROM)
		//-------------------------------------
			// CMF_A
#define IMMR_CMFMCR_A              0x0800
#define IMMR_CMFTST_A              0x0804
#define IMMR_CMFCTL_A              0x0808
	
			// CMF_B
#define IMMR_CMFMCR_B              0x0840
#define IMMR_CMFTST_B              0x0844
#define IMMR_CMFCTL_B              0x0848

		//-------------------------------------
		// DPTRAM (Dual-Port TPU RAM)
		//-------------------------------------
#define IMMR_DPTMCR                0x4000
#define IMMR_RAMTST                0x4002
#define IMMR_RAMBAR                0x4004
#define IMMR_MISRH                 0x4006
#define IMMR_MISRL                 0x4008
#define IMMR_MISCNT                0x400a

		//-------------------------------------
		// TPU3 (Time processing unit)
		//-------------------------------------
			// TPU-A
#define IMMR_TPUMCR_A              0x8000
#define IMMR_TCR_A                 0x8002
#define IMMR_DSCR_A                0x8004
#define IMMR_DSSR_A                0x8006
#define IMMR_TICR_A                0x8008
#define IMMR_CIER_A                0x800a
#define IMMR_CFSR0_A               0x800c
#define IMMR_CFSR1_A               0x800e
#define IMMR_CFSR2_A               0x8010
#define IMMR_CFSR3_A               0x8012
#define IMMR_HSQR0_A               0x8014
#define IMMR_HSQR1_A               0x8016
#define IMMR_HSRR0_A               0x8018
#define IMMR_HSRR1_A               0x801a
#define IMMR_CPR0_A                0x801c
#define IMMR_CPR1_A                0x801e
#define IMMR_CISR_A                0x8020
#define IMMR_LR_A                  0x8022
#define IMMR_SGLR_A                0x8024
#define IMMR_DCNR_A                0x8026
#define IMMR_TPUMCR2_A             0x8028
#define IMMR_TPUMCR3_A             0x802a
#define IMMR_ISDR_A                0x802c
#define IMMR_ISCR_A                0x802e

              		// TPU-B
#define IMMR_TPUMCR_B              0x8400
#define IMMR_TCR_B                 0x8402
#define IMMR_DSCR_B                0x8404
#define IMMR_DSSR_B                0x8406
#define IMMR_TICR_B                0x8408
#define IMMR_CIER_B                0x840a
#define IMMR_CFSR0_B               0x840c
#define IMMR_CFSR1_B               0x840e
#define IMMR_CFSR2_B               0x8410
#define IMMR_CFSR3_B               0x8412
#define IMMR_HSQR0_B               0x8414
#define IMMR_HSQR1_B               0x8416
#define IMMR_HSRR0_B               0x8418
#define IMMR_HSRR1_B               0x841a
#define IMMR_CPR0_B                0x841c
#define IMMR_CPR1_B                0x841e
#define IMMR_CISR_B                0x8420
#define IMMR_LR_B                  0x8422
#define IMMR_SGLR_B                0x8424
#define IMMR_DCNR_B                0x8426
#define IMMR_TPUMCR2_B             0x8428
#define IMMR_TPUMCR3_B             0x842a
#define IMMR_ISDR_B                0x842c
#define IMMR_ISCR_B                0x842e


		//-------------------------------------
		// QADC64 (Queued Analog-to-digital Converter)
		//-------------------------------------
			// QUADC-A
#define IMMR_QUADC64MCR_A          0x8800
#define IMMR_QUADC64TEST_A         0x8802
#define IMMR_QUADC64INT_A          0x8804
#define IMMR_PORTQA_A_PORTQB_A     0x8806
#define IMMR_DDRQA_A_DDRQB_A       0x8808
#define IMMR_QUACR0_A              0x880a
#define IMMR_QUACR1_A              0x880c
#define IMMR_QUACR2_A              0x880e
#define IMMR_QUASR0_A              0x8810
#define IMMR_QUASR1_A              0x8812
#define IMMR_CCW_A                 0x8a00
#define IMMR_RJURR_A               0x8a80
#define IMMR_LJSRR_A               0x8b00
#define IMMR_LJURR_A               0x8b80

			// QUADC-B
#define IMMR_QUADC64MCR_B          0x8c00
#define IMMR_QUADC64TEST_B         0x8c02
#define IMMR_QUADC64INT_B          0x8c04
#define IMMR_PORTQA_B_PORTQB_B     0x8c06
#define IMMR_DDRQA_B_DDRQB_B       0x8c08
#define IMMR_QUACR0_B              0x8c0a
#define IMMR_QUACR1_B              0x8c0c
#define IMMR_QUACR2_B              0x8c0e
#define IMMR_QUASR0_B              0x8c10
#define IMMR_QUASR1_B              0x8c12
#define IMMR_CCW_B                 0x8e00
#define IMMR_RJURR_B               0x8e80
#define IMMR_LJSRR_B               0x8f00
#define IMMR_LJURR_B               0x8f80

		//-------------------------------------
		// QSMCM (Queued Serial Multi-Channel Module)
		//-------------------------------------
#define IMMR_QSMCMMCR              0x9000
#define IMMR_QTEST                 0x9002
#define IMMR_QDSCI_IL              0x9004
#define IMMR_QSPI_IL               0x9006
#define IMMR_SCC1R0                0x9008
#define IMMR_SCC1R1                0x900a
#define IMMR_SC1SR                 0x900c
#define IMMR_SC1DR                 0x900e
#define IMMR_PORTQS                0x9014
#define IMMR_PQSPAR_DDRQST         0x9016
#define IMMR_SPCR0                 0x9018
#define IMMR_SPCR1                 0x901a
#define IMMR_SPCR2                 0x901c
#define IMMR_SPCR3                 0x901e
#define IMMR_SPSR                  0x901f
#define IMMR_SCC2R0                0x9020
#define IMMR_SCC2R1                0x9022
#define IMMR_SC2SR                 0x9024
#define IMMR_SC2DR                 0x9026
#define IMMR_QSCI1CR               0x9028
#define IMMR_QSCI1SR               0x902a
#define IMMR_SCTQ                  0x902c
#define IMMR_SCRQ                  0x904c
#define IMMR_RECRAM                0x9140
#define IMMR_TRAN_RAM              0x9180
#define IMMR_COMD_RAM              0x91c0

		//-------------------------------------
		// MIOS1 (Modular Input/Output Subsystem)
		//-------------------------------------
			// MPIOS Pulse width modulation submodule 0
#define IMMR_MPWMSMPERR_0          0xa000
#define IMMR_MPWMSMPULR_0          0xa002
#define IMMR_MPWMSMCNTR_0          0xa004
#define IMMR_MPWMSMSCR_0           0xa006
		
			// MPIOS Pulse width modulation submodule 1
#define IMMR_MPWMSMPERR_1          0xa008
#define IMMR_MPWMSMPULR_1          0xa00a
#define IMMR_MPWMSMCNTR_1          0xa00c
#define IMMR_MPWMSMSCR_1           0xa00e

			// MPIOS Pulse width modulation submodule 2
#define IMMR_MPWMSMPERR_2          0xa010
#define IMMR_MPWMSMPULR_2          0xa012
#define IMMR_MPWMSMCNTR_2          0xa014
#define IMMR_MPWMSMSCR_2           0xa016

			// MPIOS Pulse width modulation submodule 3
#define IMMR_MPWMSMPERR_3          0xa018
#define IMMR_MPWMSMPULR_3          0xa01a
#define IMMR_MPWMSMCNTR_3          0xa01c
#define IMMR_MPWMSMSCR_3           0xa01e

			// MIOS Modulus counter submodule 6
#define IMMR_MMCSMCNT_6            0xa030
#define IMMR_MMCSMML_6             0xa032
#define IMMR_MMCSMSCRD_6           0xa034
#define IMMR_MMCSMSCR_6            0xa036

			// MIOS Double Action submodule 11
#define IMMR_MDASMAR_11            0xa058
#define IMMR_MDASMBR_11            0xa05a
#define IMMR_MDASMSCRD_11          0xa05c
#define IMMR_MDASMSCR_11           0xa05e

			// MIOS Double Action submodule 12
#define IMMR_MDASMAR_12            0xa060
#define IMMR_MDASMBR_12            0xa062
#define IMMR_MDASMSCRD_12          0xa064
#define IMMR_MDASMSCR_12           0xa066

			// MIOS Double Action submodule 13
#define IMMR_MDASMAR_13            0xa068
#define IMMR_MDASMBR_13            0xa06a
#define IMMR_MDASMSCRD_13          0xa06c
#define IMMR_MDASMSCR_13           0xa06e

			// MIOS Double Action submodule 14
#define IMMR_MDASMAR_14            0xa070
#define IMMR_MDASMBR_14            0xa072
#define IMMR_MDASMSCRD_14          0xa074
#define IMMR_MDASMSCR_14           0xa076

			// MIOS Double Action submodule 15
#define IMMR_MDASMAR_15            0xa078
#define IMMR_MDASMBR_15            0xa07a
#define IMMR_MDASMSCRD_15          0xa07c
#define IMMR_MDASMSCR_15           0xa07e

			// MPIOS Pulse width modulation submodule 16
#define IMMR_MPWMSMPERR_16         0xa080
#define IMMR_MPWMSMPULR_16         0xa082
#define IMMR_MPWMSMCNTR_16         0xa084
#define IMMR_MPWMSMSCR_16          0xa086

			// MPIOS Pulse width modulation submodule 17
#define IMMR_MPWMSMPERR_17         0xa088
#define IMMR_MPWMSMPULR_17         0xa08a
#define IMMR_MPWMSMCNTR_17         0xa08c
#define IMMR_MPWMSMSCR_17          0xa08e

			// MPIOS Pulse width modulation submodule 18
#define IMMR_MPWMSMPERR_18         0xa090
#define IMMR_MPWMSMPULR_18         0xa092
#define IMMR_MPWMSMCNTR_18         0xa094
#define IMMR_MPWMSMSCR_18          0xa096

			// MPIOS Pulse width modulation submodule 19
#define IMMR_MPWMSMPERR_19         0xa098
#define IMMR_MPWMSMPULR_19         0xa09a
#define IMMR_MPWMSMCNTR_19         0xa09c
#define IMMR_MPWMSMSCR_19          0xa09e

			// MIOS Modulus counter submodule 22
#define IMMR_MMCSMCNT_22           0xa0b0
#define IMMR_MMCSMML_22            0xa0b2
#define IMMR_MMCSMSCRD_22          0xa0b4
#define IMMR_MMCSMSCR_22           0xa0b6

			// MIOS Double action submodule 27
#define IMMR_MDASMAR_27            0xa0d8
#define IMMR_MDASMBR_27            0xa0da
#define IMMR_MDASMSCRD_27          0xa0dc
#define IMMR_MDASMSCR_27           0xa0de

			// MIOS Double action submodule 28
#define IMMR_MDASMAR_28            0xa0e0
#define IMMR_MDASMBR_28            0xa0e2
#define IMMR_MDASMSCRD_28          0xa0e4
#define IMMR_MDASMSCR_28           0xa0e6

			// MIOS Double action submodule 29
#define IMMR_MDASMAR_29            0xa0e8
#define IMMR_MDASMBR_29            0xa0ea
#define IMMR_MDASMSCRD_29          0xa0ec
#define IMMR_MDASMSCR_29           0xa0ee

			// MIOS Double action submodule 30
#define IMMR_MDASMAR_30            0xa0f0
#define IMMR_MDASMBR_30            0xa0f2
#define IMMR_MDASMSCRD_30          0xa0f4
#define IMMR_MDASMSCR_30           0xa0f6

			// MIOS Double action submodule 31
#define IMMR_MDASMAR_31            0x0af8
#define IMMR_MDASMBR_31            0xa0fa
#define IMMR_MDASMSCRD_31          0xa0fc
#define IMMR_MDASMSCR_31           0xa0fe

			// MIOS Paralell port I/O submodule
#define IMMR_MPIOSMDR              0xa100
#define IMMR_MPIOSMDDR             0xa102

			// MIOS Bus interface Submodule
#define IMMR_MIOS1TPCR             0xa800
#define IMMR_MIOS1VNR              0xa802
#define IMMR_MIOS1MCR              0xa806

			// MIOS Counter / Prescaler submodule
#define IMMR_MCPSMSCR              0xa816

			// MIOS Interrupt request submodule 0
#define IMMR_MIOS1SR0              0xac00
#define IMMR_MIOS1ER0              0xac04
#define IMMR_MIOS1RPR0             0xac06
#define IMMR_MIOS1LVL0             0xac30

			// Mios Interrupt request submodule 1
#define IMMR_MIOS1SR1              0xac40
#define IMMR_MIOS1ER1              0xac44
#define IMMR_MIOS1RPR1             0xac46
#define IMMR_MIOS1LVL1             0xac70

		//-------------------------------------
		// TouCAN (CAN 2.0B Controller)
		//-------------------------------------
			// TouCAN_A
#define IMMR_TCNMCR_A              0xb080
#define IMMR_TTR_A                 0xb082
#define IMMR_CANICR_A              0xb084
#define IMMR_CANCTRL0_A_CANCTRL1_A 0xb086
#define IMMR_PRESDIV_A_CTRL2_A     0xb088
#define IMMR_TIMER_A               0xb08a
#define IMMR_RXGMASKHI_A           0xb090
#define IMMR_RXGMASKLO_A           0xb092
#define IMMR_RX14MASKHI_A          0xb094
#define IMMR_RX14MASKLO_A          0xb096
#define IMMR_RX15MASKHI_A          0xb098
#define IMMR_RX15MASKLO_A          0xb09a
#define IMMR_ESTAT_A               0xb0a0
#define IMMR_IMASK_A               0xb0a2
#define IMMR_IFLAG_A               0xb0a4
#define IMMR_RXECTR_A_TXECTR_A     0xb0a6

			// TouCAN_B
#define IMMR_TCNMCR_B              0xb480
#define IMMR_TTR_B                 0xb482
#define IMMR_CANICR_B              0xb484
#define IMMR_CANCTRL0_B_CANCTRL1_B 0xb486
#define IMMR_PRESDIV_B_CTRL2_B     0xb488
#define IMMR_TIMER_B               0xb48a
#define IMMR_RXGMASKHI_B           0xb490
#define IMMR_RXGMASKLO_B           0xb492
#define IMMR_RX14MASKHI_B          0xb494
#define IMMR_RX14MASKLO_B          0xb496
#define IMMR_RX15MASKHI_B          0xb498
#define IMMR_RX15MASKLO_B          0xb49a
#define IMMR_ESTAT_B               0xb4a0
#define IMMR_IMASK_B               0xb4a2
#define IMMR_IFLAG_B               0xb4a4
#define IMMR_RXECTR_A_TXECTR_B     0xb4a6

		//-------------------------------------
		// UIMB (U-Bus to IMB3 Bus Interface)
		//-------------------------------------
#define IMMR_UMCR                  0xbf80
#define IMMR_UTSTCREG              0xbf90
#define IMMR_UIPEND                0xbfa0

		//-------------------------------------
		// SRAM (Static RAM Access memory)
		//-------------------------------------
#define IMMR_SRAMMCR_A             0x84000
#define IMMR_SRAMTST_A             0x84004
#define IMMR_SRAMMCR_B             0x84008
#define IMMR_SRAMTST_B             0x8400c

#endif // CYGONCE_HAL_IMMR_H

