#ifndef MPC555_ISRSYS_H
#define MPC555_ISRSYS_H

// This file is meant for configuring the interrupt system on the MPC555 microcontroller.
// Setting interrupt priorities should be done here. Everything that needs to be done in the
// user code if an interrupt is wanted is creating the Cyg_Interrupt object and unmasking
// the associated interrupt

//---------------------------------
// Includes and frward declarations
//---------------------------------
#include "pkgconf/system.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)

//-------------------------
// The SIU interrupt levels
//-------------------------
#define CYG_HAL_ISR_SIU_LEVEL_0                          0
#define CYG_HAL_ISR_SIU_LEVEL_1                          1
#define CYG_HAL_ISR_SIU_LEVEL_2                          2
#define CYG_HAL_ISR_SIU_LEVEL_3                          3
#define CYG_HAL_ISR_SIU_LEVEL_4                          4
#define CYG_HAL_ISR_SIU_LEVEL_5                          5
#define CYG_HAL_ISR_SIU_LEVEL_6                          6
#define CYG_HAL_ISR_SIU_LEVEL_7                          7

//----------------------------------------------------------
// The IMB3 Interrupt priorities.
// Notice that priorites 0..6  are mapped on SIU levels 0..6
//                       7..31 are mapped on SIU level  7
//----------------------------------------------------------
#define CYG_HAL_ISR_IMB3_PRIORITY_0                      0
#define CYG_HAL_ISR_IMB3_PRIORITY_1                      1
#define CYG_HAL_ISR_IMB3_PRIORITY_2                      2
#define CYG_HAL_ISR_IMB3_PRIORITY_3                      3
#define CYG_HAL_ISR_IMB3_PRIORITY_4                      4
#define CYG_HAL_ISR_IMB3_PRIORITY_5                      5
#define CYG_HAL_ISR_IMB3_PRIORITY_6                      6
#define CYG_HAL_ISR_IMB3_PRIORITY_7                      7
#define CYG_HAL_ISR_IMB3_PRIORITY_8                      8
#define CYG_HAL_ISR_IMB3_PRIORITY_9                      9
#define CYG_HAL_ISR_IMB3_PRIORITY_10                    10
#define CYG_HAL_ISR_IMB3_PRIORITY_11                    11
#define CYG_HAL_ISR_IMB3_PRIORITY_12                    12
#define CYG_HAL_ISR_IMB3_PRIORITY_13                    13
#define CYG_HAL_ISR_IMB3_PRIORITY_14                    14
#define CYG_HAL_ISR_IMB3_PRIORITY_15                    15
#define CYG_HAL_ISR_IMB3_PRIORITY_16                    16
#define CYG_HAL_ISR_IMB3_PRIORITY_17                    17
#define CYG_HAL_ISR_IMB3_PRIORITY_18                    18
#define CYG_HAL_ISR_IMB3_PRIORITY_19                    19
#define CYG_HAL_ISR_IMB3_PRIORITY_20                    20
#define CYG_HAL_ISR_IMB3_PRIORITY_21                    21
#define CYG_HAL_ISR_IMB3_PRIORITY_22                    22
#define CYG_HAL_ISR_IMB3_PRIORITY_23                    23
#define CYG_HAL_ISR_IMB3_PRIORITY_24                    24
#define CYG_HAL_ISR_IMB3_PRIORITY_25                    25
#define CYG_HAL_ISR_IMB3_PRIORITY_26                    26
#define CYG_HAL_ISR_IMB3_PRIORITY_27                    27
#define CYG_HAL_ISR_IMB3_PRIORITY_28                    28
#define CYG_HAL_ISR_IMB3_PRIORITY_29                    29
#define CYG_HAL_ISR_IMB3_PRIORITY_30                    30
#define CYG_HAL_ISR_IMB3_PRIORITY_31                    31

// The interrupt sources that directly interrupt the SIU
// These are the levels that must be assigned when creating the isr_dispatchers
#define ISR_DISPATCHER_LEVEL_PIT                        CYG_HAL_ISR_SIU_LEVEL_0
#define ISR_DISPATCHER_LEVEL_PLL                        CYG_HAL_ISR_SIU_LEVEL_1
#define ISR_DISPATCHER_LEVEL_TB                         CYG_HAL_ISR_SIU_LEVEL_2
#define ISR_DISPATCHER_LEVEL_IMB3_0                     CYG_HAL_ISR_SIU_LEVEL_3
#define ISR_DISPATCHER_LEVEL_IMB3_1                     CYG_HAL_ISR_SIU_LEVEL_4
#define ISR_DISPATCHER_LEVEL_IMB3_2                     CYG_HAL_ISR_SIU_LEVEL_5
#define ISR_DISPATCHER_LEVEL_RTC                        CYG_HAL_ISR_SIU_LEVEL_6
#define ISR_DISPATCHER_LEVEL_IMB3_3                     CYG_HAL_ISR_SIU_LEVEL_7

// The interrupt source priorities. These are the values that must be used when
// creating the isr sources
#define ISR_SOURCE_PRIORITY_PIT                         CYG_HAL_ISR_SIU_LEVEL_0
#define ISR_SOURCE_PRIORITY_PLL                         CYG_HAL_ISR_SIU_LEVEL_1
#define ISR_SOURCE_PRIORITY_TB                          CYG_HAL_ISR_SIU_LEVEL_2
#define ISR_SOURCE_PRIORITY_TPU_A                       CYG_HAL_ISR_IMB3_PRIORITY_3
#define ISR_SOURCE_PRIORITY_TOUCAN_A                    CYG_HAL_ISR_IMB3_PRIORITY_4
#define ISR_SOURCE_PRIORITY_MIOS_A                      CYG_HAL_ISR_IMB3_PRIORITY_5
#define ISR_SOURCE_PRIORITY_RTC                         CYG_HAL_ISR_SIU_LEVEL_6
#define ISR_SOURCE_PRIORITY_QUADC_A_QUEUE1              CYG_HAL_ISR_IMB3_PRIORITY_8
#define ISR_SOURCE_PRIORITY_QUADC_A_QUEUE2              CYG_HAL_ISR_IMB3_PRIORITY_9
#define ISR_SOURCE_PRIORITY_QUADC_B_QUEUE1              CYG_HAL_ISR_IMB3_PRIORITY_10
#define ISR_SOURCE_PRIORITY_QUADC_B_QUEUE2              CYG_HAL_ISR_IMB3_PRIORITY_11
#define ISR_SOURCE_PRIORITY_TPU_B                       CYG_HAL_ISR_IMB3_PRIORITY_12
#define ISR_SOURCE_PRIORITY_TOUCAN_B                    CYG_HAL_ISR_IMB3_PRIORITY_13
#define ISR_SOURCE_PRIORITY_MIOS_B                      CYG_HAL_ISR_IMB3_PRIORITY_14
#define ISR_SOURCE_PRIORITY_QSPI                        CYG_HAL_ISR_IMB3_PRIORITY_30
#define ISR_SOURCE_PRIORITY_QSCI                        CYG_HAL_ISR_IMB3_PRIORITY_31

// The interrupt levels. These are the values that must be used when creating the 
// Cyg_Interrupt objects
// The external interrupts cannot be assigned any priority
// hardware takes care of that
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ0_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ1_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ2_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ3_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ4_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ5_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ6_PRIORITY          0
#define CYGNUM_HAL_INTERRUPT_SIU_IRQ7_PRIORITY          0

#define CYGNUM_HAL_INTERRUPT_SIU_TB_A_PRIORITY          ISR_SOURCE_PRIORITY_TB
#define CYGNUM_HAL_INTERRUPT_SIU_TB_B_PRIORITY          ISR_SOURCE_PRIORITY_TB
#define CYGNUM_HAL_INTERRUPT_SIU_PIT_PRIORITY           ISR_SOURCE_PRIORITY_PIT
#define CYGNUM_HAL_INTERRUPT_SIU_RTC_SEC_PRIORITY       ISR_SOURCE_PRIORITY_RTC
#define CYGNUM_HAL_INTERRUPT_SIU_RTC_ALR_PRIORITY       ISR_SOURCE_PRIORITY_RTC
#define CYGNUM_HAL_INTERRUPT_SIU_COL_PRIORITY           ISR_SOURCE_PRIORITY_PLL
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCA_CI1_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_A_QUEUE1
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCA_PI1_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_A_QUEUE1
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCA_CI2_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_A_QUEUE2
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCA_PI2_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_A_QUEUE2
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCB_CI1_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_B_QUEUE1
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCB_PI1_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_B_QUEUE1
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCB_CI2_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_B_QUEUE2
#define CYGNUM_HAL_INTERRUPT_IMB3_QUADCB_PI2_PRIORITY   ISR_SOURCE_PRIORITY_QUADC_B_QUEUE2
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI0_TX_PRIORITY      ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI0_TXC_PRIORITY     ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI0_RX_PRIORITY      ISR_SOURCE_PRIORITY_QSCI 
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI0_IDLE_PRIORITY    ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TX_PRIORITY      ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC_PRIORITY     ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_RX_PRIORITY      ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_IDLE_PRIORITY    ISR_SOURCE_PRIORITY_QSCI 
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_RXQTHF_PRIORITY  ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_RXQBHF_PRIORITY  ISR_SOURCE_PRIORITY_QSCI  
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXQTHE_PRIORITY  ISR_SOURCE_PRIORITY_QSCI 
#define CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXQBHE_PRIORITY  ISR_SOURCE_PRIORITY_QSCI
#define CYGNUM_HAL_INTERRUPT_IMB3_SPI_FI_PRIORITY       ISR_SOURCE_PRIORITY_QSPI
#define CYGNUM_HAL_INTERRUPT_IMB3_SPI_MODF_PRIORITY     ISR_SOURCE_PRIORITY_QSPI
#define CYGNUM_HAL_INTERRUPT_IMB3_SPI_HALTA_PRIORITY    ISR_SOURCE_PRIORITY_QSPI
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_BOFF_PRIORITY ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_ERR_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_WU_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B0_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B1_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B2_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B3_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B4_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B5_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B6_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B7_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B8_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B9_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B10_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B11_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B12_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B13_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B14_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B15_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_BOFF_PRIORITY ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_ERR_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_WU_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B0_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B1_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B2_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B3_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B4_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B5_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B6_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B7_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B8_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B9_PRIORITY   ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B10_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B11_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B12_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B13_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B14_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B15_PRIORITY  ISR_SOURCE_PRIORITY_TOUCAN_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH0_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH1_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH2_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH3_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH4_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH5_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH6_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH7_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH8_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH9_PRIORITY     ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH10_PRIORITY    ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH11_PRIORITY    ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH12_PRIORITY    ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH13_PRIORITY    ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH14_PRIORITY    ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUA_CH15_PRIORITY    ISR_SOURCE_PRIORITY_TPU_A
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH0_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH1_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH2_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH3_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH4_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH5_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH6_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH7_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH8_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH9_PRIORITY     ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH10_PRIORITY    ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH11_PRIORITY    ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH12_PRIORITY    ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH13_PRIORITY    ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH14_PRIORITY    ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_TPUB_CH15_PRIORITY    ISR_SOURCE_PRIORITY_TPU_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM0_PRIORITY   ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM1_PRIORITY   ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM2_PRIORITY   ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM3_PRIORITY   ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MMCSM6_PRIORITY  ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM11_PRIORITY ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM12_PRIORITY ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM13_PRIORITY ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM14_PRIORITY ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM15_PRIORITY ISR_SOURCE_PRIORITY_MIOS_A
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM16_PRIORITY  ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM17_PRIORITY  ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM18_PRIORITY  ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MPWM19_PRIORITY  ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MMCSM22_PRIORITY ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM27_PRIORITY ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM28_PRIORITY ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM29_PRIORITY ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM30_PRIORITY ISR_SOURCE_PRIORITY_MIOS_B
#define CYGNUM_HAL_INTERRUPT_IMB3_MIOS_MDASM31_PRIORITY ISR_SOURCE_PRIORITY_MIOS_B

#endif // if !defined ...

#endif // MPC555_ISRSYS_H
