#ifndef IMB3_ISRDISPATCH_CXX
#define IMB3_ISRDISPATCH_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/hal/imb3_isrdispatch.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/hal_intr.h"
#include "cyg/hal/imb3_isrsource.h"

//-----------------------------------
// Public constructors and destructor
//-----------------------------------
IMB3_isrDispatcher::IMB3_isrDispatcher()
    : HAL_isrDispatcher(),
      m_siulevel(255)   // Initialise with something impossible
{
  int count;
  for(count = 0; count < 32; count++)
    m_sources[count] = 0;

  // Make sure the IMB3 interrupt controller supports 32 interrupt levels
  cyg_uint32 umcr;
  HAL_READ_UINT32(CYGARC_REG_IMM_UMCR, umcr);
  umcr |= 0x60000000;
  HAL_WRITE_UINT32(CYGARC_REG_IMM_UMCR, umcr);
}

// Provide an (empty) virtual destructor
IMB3_isrDispatcher::~IMB3_isrDispatcher()
{
}

// attachToPriority
bool IMB3_isrDispatcher::attachToPriority(IMB3_isrSource * a_source, unsigned char a_priority)
{
  CYG_ASSERT(a_priority < 32, "Interrupt priorities 0..31");
  if(!m_sources[a_priority])
  {
    m_sources[a_priority] = a_source;
   
    return true;
  }

  return false;
}

//------------------------------------------
// Protected member functions implementation
//------------------------------------------
// setInterruptingLevel
bool IMB3_isrDispatcher::setInterruptingLevel(unsigned char a_level)
{ 
  m_siulevel = a_level;
  
  // Notice, the parameter is the SIU level, so, a number in the range 0..7
  bool result = true;
  int count = 0;

  // Makesure the requested level can be mapped on the registered
  // interrupt sources.
  if(a_level < 7)
  { // IMB3 levels 0-6 are mapped to SIU levels 0-6
    while((count < 32) && result)
    {
      if(count != a_level)
        result = (m_sources[count] == 0);

      count++; 
    }

    // So far so good, check for a source on the level
    if(result)
      result = (m_sources[a_level] != 0);

    return result;
  }

  if(a_level == 7)
  { // IMB3 levels 7-31 are mapped to SIU level 7
    while((count < 7) && result)
    {
      result = (m_sources[count] == 0);
      count++;
    }

    // So far so good, check that there is at least one source registered
    if(result)
    {
      result = false;
      while((count < 32) && !result)
      {
        result = (m_sources[count] != 0);
        count++;
      }
    }

    return result;
  }

  return false;    
}

// enableInterruptingDevice
void IMB3_isrDispatcher::enableInterruptingDevice(void)
{
  int count;
  for(count = 0; count < 32; count++)
  {
    if(m_sources[count])
      m_sources[count]->enableInterrupts();
  }
}

// disableInterruptingDevice
void IMB3_isrDispatcher::disableInterruptingDevice(void)
{
  int count;
  for(count = 0; count < 32; count++)
  {
    if(m_sources[count])
      m_sources[count]->disableInterrupts();
  }
}

// findInterruptingVector
cyg_uint32 IMB3_isrDispatcher::findInterruptingVector(void)
{
  // Make it go fast, levels between 0..6 ar mapped one to one SIU-IMB3
  if(m_siulevel < 7)
    return m_sources[m_siulevel]->findInterruptingVector(m_siulevel);

  // These are the lowest priority, no big deal that we have to poll
  else
  {
    cyg_uint32 uipend;
    HAL_READ_UINT32(CYGARC_REG_IMM_UIPEND, uipend);

    unsigned char priority = 31;    
    cyg_uint32 mask = 0x00000001;

    while(priority > 6) // Must be 7..31
    {
      if(uipend & mask)
        return m_sources[priority]->findInterruptingVector(priority);
      else
      {
        mask = mask << 1;
        priority--;
      }
    }
  }

  return 0;
}
#endif // if !defined ...

#endif // IMB3_ISRDISPATCH_CXX
