/*==========================================================================
//
//	mpc555.ld
//
//	Linker script for MPC555
//
//========================================================================*/

STARTUP(vectors.o)
ENTRY(__reset_vector)

#ifdef EXTRAS
INPUT(extras.o)
#endif

GROUP(libtarget.a libgcc.a)

#define ALIGN_LMA 8
#define FOLLOWING(_section_) AT ((LOADADDR (_section_) + SIZEOF (_section_) + ALIGN_LMA - 1) & ~ (ALIGN_LMA - 1))
#define LMA_EQ_VMA
#define FORCE_OUTPUT . = .

#include <pkgconf/system.h>

#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
#define LOADATTR
#else
#define LOADATTR (NOLOAD)
#endif

#define SECTIONS_BEGIN

#define SECTION_vectors(_region_, _vma_, _lma_) \
  .vectors _vma_ : _lma_ \
  { \
    FORCE_OUTPUT; \
    KEEP(*(.vectors)) \
  } > _region_

#define SECTION_text(_region_, _vma_, _lma_) \
  .text _vma_ : _lma_ \
  { \
    _stext = .; \
    *(.text*) \
    *(.gnu.warning) \
    *(.gnu.linkonce*) \
    *(.init) \
  } > _region_ \
  _etext = .; \
  PROVIDE (etext = .);

#define SECTION_fini(_region_, _vma_, _lma_) \
  .fini _vma_ : _lma_ \
  { \
    FORCE_OUTPUT; \
    *(.fini) \
  } > _region_

#define SECTION_rodata1(_region_, _vma_, _lma_) \
  .rodata1 _vma_ : _lma_ \
  { \
    FORCE_OUTPUT; \
    *(.rodata1) \
  } > _region_

#define SECTION_rodata(_region_, _vma_, _lma_) \
  .rodata _vma_ : _lma_ \
  { \
    FORCE_OUTPUT; \
    *(.rodata*) \
  } > _region_

#define SECTION_fixup(_region_, _vma_, _lma_) \
  .fixup _vma_ : _lma_ \
  { \
     __FIXUP_START__ = ABSOLUTE(.); \
     *(.fixup) \
     __FIXUP_END__ = ABSOLUTE(.); \
  } > _region_

#define SECTION_gcc_except_table(_region_, _vma_, _lma_) \
  .gcc_except_table _vma_ : _lma_ \
  { \
    __EXCEPT_START__ = ABSOLUTE(.); \
    *(.gcc_except_table) \
    __EXCEPT_END__ = ABSOLUTE(.); \
  } > _region_

#define SECTION_data(_region_, _vma_, _lma_) \
  .data _vma_ : _lma_ \
  { \
    __ram_data_start = ABSOLUTE(.); \
    *(.data*) \
    __GOT1_START__ = ABSOLUTE(.); \
    *(.got1) \
    __GOT1_END__ = ABSOLUTE(.); \
    /* Put .ctors and .dtors next to the .got2 section, so that */    \
    /* the pointers get relocated with -mrelocatable.           */    \
    . = ALIGN(8); \
    __CTOR_LIST__ = ABSOLUTE(.); \
    KEEP(*(SORT(.ctors*))) \
    __CTOR_END__ = ABSOLUTE(.); \
    __DTOR_LIST__ = ABSOLUTE(.); \
    KEEP(*(SORT(.dtors*))) \
    __DTOR_END__ = ABSOLUTE(.); \
    . = ALIGN(8); \
    __DEVTAB__ = ABSOLUTE (.); \
    KEEP(*(SORT(.devtab*))) \
    __DEVTAB_END__ = ABSOLUTE (.); \
    __GOT2_START__ = ABSOLUTE(.); \
    *(.got2) \
    __GOT2_END__ = ABSOLUTE(.); \
    __GOT_START = ABSOLUTE(.); \
    _GLOBAL_OFFSET_TABLE_ = ABSOLUTE(. + 32768); \
    _SDA_BASE_ = ABSOLUTE(.); \
    *(.got.plt) \
    *(.got) \
    __GOT_END__ = ABSOLUTE(.); \
    *(.dynamic) \
    /* We want the small data sections together, so single-instruction */   \
    /* offsets can access them all, and initialized data all before    */   \
    /* uninitialized, so we can shorten the on-disk segment size.      */   \
    __SDATA_START__ = ABSOLUTE(.); \
    *(.sdata) \
    __SDATA2_START__ = ABSOLUTE(.); \
    *(.sdata2) \
  } > _region_ \
  __rom_data_start = LOADADDR(.data); \
  __ram_data_end = .; \
  PROVIDE(__ram_data_end = .); \
  _edata = .; \
  PROVIDE (_edata = .);

#define SECTION_exc_vsr(_region_, _vma_, _lma_) \
  .exc_vsr _vma_ LOADATTR : _lma_ \
  { \
     __ram_exc_vsr_data_start = ABSOLUTE(.); \
     *(.exc_vsr) \
  } > _region_ \
  __rom_exc_vsr_data_start = LOADADDR(.exc_vsr); \
  __ram_exc_vsr_data_end = .;

#define SECTION_isr_handlers(_region_, _vma_, _lma_) \
  .isr_handlers _vma_ LOADATTR : _lma_ \
  { \
    __ram_isr_handlers_data_start = ABSOLUTE(.); \
    *(.isr_handlers) \
  } > _region_ \
  __rom_isr_handlers_data_start = LOADADDR(.isr_handlers); \
  __ram_isr_handlers_data_end = .;

#define SECTION_isr_data(_region_, _vma_, _lma_) \
  .isr_data _vma_ LOADATTR : _lma_ \
  { \
    __ram_isr_data_data_start = ABSOLUTE(.); \
    *(.isr_data); \
  } > _region_ \
  __rom_isr_data_data_start = LOADADDR(.isr_data); \
  __ram_isr_data_data_end = .;

#define SECTION_isr_obj(_region_, _vma_, _lma_) \
  .isr_obj _vma_ LOADATTR : _lma_ \
  { \
    __ram_isr_obj_data_start = ABSOLUTE(.); \
    *(.isr_obj) \
  } > _region_ \
  __rom_isr_obj_data_start = LOADADDR(.isr_obj); \
  __ram_isr_obj_data_end = .;

#define SECTION_sbss(_region_, _vma_, _lma_) \
  .sbss _vma_ : _lma_ \
  { \
     __sbss_start = ABSOLUTE (.); \
     __SBSS_START__ = ABSOLUTE(.); \
     *(.sbss) \
      __SBSS_END__ = ABSOLUTE(.); \
     __SBSS2_START__ = ABSOLUTE(.); \
     *(.sbss2) \
     __SBSS2_END__ = ABSOLUTE(.); \
     *(.scommon) \
     __sbss_end = ABSOLUTE (.); \
  } > _region_

#define SECTION_bss(_region_, _vma_, _lma_) \
  .bss _vma_ : _lma_ \
  { \
    __bss_start = ABSOLUTE (.); \
    FORCE_OUTPUT; \
    *(.dynbss) \
    *(.bss) \
    *(COMMON) \
    __bss_end = ABSOLUTE (.); \
  } > _region_

#define SECTION_isrstack(_region_, _vma_, _lma_) \
  .isrstack (NOLOAD) _vma_ : _lma_ \
  { \
    . = ALIGN(4); \
    *(.isrstack) \
  } > _region_

#define SECTION_immr(_region_, _vma_, _lma_) \
  .immr (NOLOAD) _vma_ : _lma_ \
  { \
    *(.immr) \
  } > _region_

#define SECTIONS_END \
  . = ALIGN(4); \
   _end = .; \
  PROVIDE (_end = .);

#include CYGHWR_MEMORY_LAYOUT_LDI
