#ifndef PIT_ISRDISPATCH_CXX
#define PIT_ISRDISPATCH_CXX

//----------------------------------
// includes and forward declarations
//----------------------------------
#include "cyg/hal/pit_isrdispatch.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/hal_io.h"
#include "cyg/hal/hal_intr.h"

//---------------------
// Class implementation
//---------------------
// Public constructors(s) and destructor
PIT_isrDispatcher::PIT_isrDispatcher()
    : HAL_isrDispatcher()
{
}

// Provide an (empty) virtual destructor
PIT_isrDispatcher::~PIT_isrDispatcher()
{
}

//------------------------------------------
// Protected member functions implementation
//------------------------------------------
// findInterruptingVector
cyg_uint32 PIT_isrDispatcher::findInterruptingVector(void)
{
  cyg_uint16 piscr;

  HAL_READ_UINT16(CYGARC_REG_IMM_PISCR, piscr);
  if((piscr & CYGARC_REG_IMM_PISCR_PS) && (piscr & CYGARC_REG_IMM_PISCR_PIE))
    return CYGNUM_HAL_INTERRUPT_SIU_PIT;

  return 0;
}

// setInterruptingLevel
bool PIT_isrDispatcher::setInterruptingLevel(unsigned char a_level)
{
  cyg_uint16 piscr;
  cyg_uint16 level = (CYGARC_REG_IMM_PISCR_IRQ0 >> a_level);

  HAL_READ_UINT16(CYGARC_REG_IMM_PISCR, piscr);
  piscr &= ~((cyg_uint16)CYGARC_REG_IMM_PISCR_IRQMASK);
  piscr |= level;
  piscr &= ~(CYGARC_REG_IMM_PISCR_PS); // Prevent from clearing interrupt flags accidently
  HAL_WRITE_UINT16(CYGARC_REG_IMM_PISCR, piscr);

  return true;
}

// enableInterruptingDevice
void PIT_isrDispatcher::enableInterruptingDevice(void)
{
  cyg_uint16 piscr;

  HAL_READ_UINT16(CYGARC_REG_IMM_PISCR, piscr);
  piscr |= CYGARC_REG_IMM_PISCR_PTE;
  piscr &= ~(CYGARC_REG_IMM_PISCR_PS); // Prevent from clearing interrupt flags accidently
  HAL_WRITE_UINT16(CYGARC_REG_IMM_PISCR, piscr);
}

// disableInterruptingDevice
void PIT_isrDispatcher::disableInterruptingDevice(void)
{
  cyg_uint16 piscr;

  HAL_READ_UINT16(CYGARC_REG_IMM_PISCR, piscr);
  piscr &= ~((cyg_uint16)CYGARC_REG_IMM_PISCR_PTE);
  piscr &= ~(CYGARC_REG_IMM_PISCR_PS); // Prevent from clearing interrupt flags accidently
  HAL_WRITE_UINT16(CYGARC_REG_IMM_PISCR, piscr);
}
#endif // if !defined

#endif // PIT_ISRDISPATCH_CXX
