#ifndef QSCI_ISRSOURCE_CXX
#define QSCI_ISRSOURCE_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/hal/qsci_isrsource.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/imb3_isrdispatch.h"
#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/hal_io.h"
#include "cyg/infra/cyg_ass.h"
#include "cyg/hal/hal_intr.h"

//-------------------------------------
// Public constructor(s) and destructor
//-------------------------------------
QSCI_isrSource::QSCI_isrSource(IMB3_isrDispatcher * a_dispatcher,
                               unsigned char a_sci_interrupt_priority,
                               unsigned char a_spi_interrupt_priority)
    : IMB3_isrSource(a_dispatcher),
      m_sci_priority(a_sci_interrupt_priority),
      m_spi_priority(a_spi_interrupt_priority)
{
  CYG_ASSERT(a_sci_interrupt_priority < 32, "Interrupt priorities 0..31");
  CYG_ASSERT(a_spi_interrupt_priority < 32, "Interrupt priorities 0..31");
  CYG_ASSERT(a_sci_interrupt_priority != a_spi_interrupt_priority, "Different isr sources on same interrupt level");

  a_dispatcher->attachToPriority(this, a_sci_interrupt_priority);
  a_dispatcher->attachToPriority(this, a_spi_interrupt_priority);

  // Now program these levels in the hardware
  cyg_uint16 qdsci_il;
  cyg_uint16 qspi_il;
  cyg_uint16 level;
  HAL_READ_UINT16(CYGARC_REG_IMM_QDSCI_IL, qdsci_il);
  HAL_READ_UINT16(CYGARC_REG_IMM_QSPI_IL, qspi_il);
  level = a_sci_interrupt_priority;
  level = (level << CYGARC_REG_IMM_QDSCI_IL_ILDSCI_SHIFT);
  level &= (CYGARC_REG_IMM_QDSCI_IL_ILDSCI);
  qdsci_il &= ~(CYGARC_REG_IMM_QDSCI_IL_ILDSCI);
  qdsci_il |= level;

  level = a_spi_interrupt_priority;
  level = (level << CYGARC_REG_IMM_QSPI_IL_ILQSPI_SHIFT);
  level &= (CYGARC_REG_IMM_QSPI_IL_ILQSPI);
  qspi_il &= ~(CYGARC_REG_IMM_QSPI_IL_ILQSPI);
  qspi_il |= level;
  HAL_WRITE_UINT16(CYGARC_REG_IMM_QDSCI_IL, qdsci_il);
  HAL_WRITE_UINT16(CYGARC_REG_IMM_QSPI_IL, qspi_il);
}

// Provide an (empty) virtual destructor
QSCI_isrSource::~QSCI_isrSource()
{
}

//------------------------
// Public member functions
//------------------------
cyg_uint32 QSCI_isrSource::findInterruptingVector(unsigned char a_priority)
{
  if(a_priority == m_sci_priority)  
  {
    cyg_uint16 status;
    cyg_uint16 control;

    HAL_READ_UINT16(CYGARC_REG_IMM_SC1SR, status);
    HAL_READ_UINT16(CYGARC_REG_IMM_SCC1R1, control);
    if((status & CYGARC_REG_IMM_SCxSR_TDRE) && (control & CYGARC_REG_IMM_SCCxR1_TIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI0_TX;
    if((status & CYGARC_REG_IMM_SCxSR_TC) && (control & CYGARC_REG_IMM_SCCxR1_TCIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI0_TXC;
    if((status & CYGARC_REG_IMM_SCxSR_RDRF) && (control & CYGARC_REG_IMM_SCCxR1_RIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI0_RX;
    if((status & CYGARC_REG_IMM_SCxSR_IDLE) && (control & CYGARC_REG_IMM_SCCxR1_ILIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI0_IDLE;
    
    HAL_READ_UINT16(CYGARC_REG_IMM_SC2SR, status);
    HAL_READ_UINT16(CYGARC_REG_IMM_SCC2R1, control);
    if((status & CYGARC_REG_IMM_SCxSR_TDRE) && (control & CYGARC_REG_IMM_SCCxR1_TIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TX;
    if((status & CYGARC_REG_IMM_SCxSR_TC) && (control & CYGARC_REG_IMM_SCCxR1_TCIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXC;
    if((status & CYGARC_REG_IMM_SCxSR_RDRF) && (control & CYGARC_REG_IMM_SCCxR1_RIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_RX;
    if((status & CYGARC_REG_IMM_SCxSR_IDLE) && (control & CYGARC_REG_IMM_SCCxR1_ILIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_IDLE;

    HAL_READ_UINT16(CYGARC_REG_IMM_QSCI1SR, status);
    HAL_READ_UINT16(CYGARC_REG_IMM_QSCI1CR, control);
    if((status & CYGARC_REG_IMM_QSCI1SR_QTHF) && (control & CYGARC_REG_IMM_QSCI1CR_QTHFI))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_RXQTHF;
    if((status & CYGARC_REG_IMM_QSCI1SR_QBHF) && (control & CYGARC_REG_IMM_QSCI1CR_QBHFI))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_RXQBHF;
    if((status & CYGARC_REG_IMM_QSCI1SR_QTHE) && (control & CYGARC_REG_IMM_QSCI1CR_QTHEI))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXQTHE;
    if((status & CYGARC_REG_IMM_QSCI1SR_QBHE) && (control & CYGARC_REG_IMM_QSCI1CR_QBHEI))
      return CYGNUM_HAL_INTERRUPT_IMB3_SCI1_TXQBHE;
  }

  if(a_priority == m_spi_priority)
  {
    cyg_uint16 spsr;
    cyg_uint16 spcr;
    HAL_READ_UINT16(CYGARC_REG_IMM_SPSR, spsr);
    HAL_READ_UINT16(CYGARC_REG_IMM_SPCR2, spcr);
    if((spsr & CYGARC_REG_IMM_SPSR_SPIF) && (spcr & CYGARC_REG_IMM_SPCR2_SPIFIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SPI_FI;

    HAL_READ_UINT16(CYGARC_REG_IMM_SPCR3, spcr);
    if((spsr & CYGARC_REG_IMM_SPSR_MODF) && (spcr & CYGARC_REG_IMM_SPCR3_HMIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SPI_MODF;

    if((spsr & CYGARC_REG_IMM_SPSR_HALTA) && (spcr & CYGARC_REG_IMM_SPCR3_HMIE))
      return CYGNUM_HAL_INTERRUPT_IMB3_SPI_HALTA;
  }

  return 0;
}

void QSCI_isrSource::enableInterrupts(void)
{
  cyg_uint16 sccxr1;
  HAL_READ_UINT16(CYGARC_REG_IMM_SCC1R1, sccxr1);
  sccxr1 |= CYGARC_REG_IMM_SCCxR1_TE;
  sccxr1 |= CYGARC_REG_IMM_SCCxR1_RE;
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SCC1R1, sccxr1);
  HAL_READ_UINT16(CYGARC_REG_IMM_SCC2R1, sccxr1);
  sccxr1 |= CYGARC_REG_IMM_SCCxR1_TE;
  sccxr1 |= CYGARC_REG_IMM_SCCxR1_RE;
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SCC2R1, sccxr1);

  cyg_uint16 spcr1;
  HAL_READ_UINT16(CYGARC_REG_IMM_SPCR1, spcr1);
  spcr1 |= CYGARC_REG_IMM_SPCR1_SPE;
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SPCR1, spcr1);
}

void QSCI_isrSource::disableInterrupts(void)
{ 
  cyg_uint16 sccxr1;
  HAL_READ_UINT16(CYGARC_REG_IMM_SCC1R1, sccxr1);
  sccxr1 &= ~(CYGARC_REG_IMM_SCCxR1_TE);
  sccxr1 &= ~(CYGARC_REG_IMM_SCCxR1_RE);
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SCC1R1, sccxr1);
  HAL_READ_UINT16(CYGARC_REG_IMM_SCC2R1, sccxr1);
  sccxr1 &= ~(CYGARC_REG_IMM_SCCxR1_TE);
  sccxr1 &= ~(CYGARC_REG_IMM_SCCxR1_RE);
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SCC2R1, sccxr1);

  cyg_uint16 spcr1;
  HAL_READ_UINT16(CYGARC_REG_IMM_SPCR1, spcr1);
  spcr1 &= ~(CYGARC_REG_IMM_SPCR1_SPE);
  HAL_WRITE_UINT16(CYGARC_REG_IMM_SPCR1, spcr1);

}
#endif // if !defined ...

#endif // QSCI_ISRSOURCE_CXX
