##===========================================================================
##
##                                 start.s
##                          MPC555 Startup code
##
##===========================================================================
## Some preprocessor stuff

#include <pkgconf/hal.h>
#include <pkgconf/system.h>

#ifdef CYGPKG_KERNEL
#include <pkgconf/kernel.h>
#endif

#ifndef CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
#define CYGARC_HAL_COMMON_EXPORT_CPU_MACROS
#endif

#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/mpc555.inc"

#define CYG_MSR_COMMON (MSR_FP | MSR_ME | MSR_RI)
#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
#define CYG_MSR CYG_MSR_COMMON
#else
#define CYG_MSR CYG_MSR_COMMON
#endif

## End of preprocessor stuff
##===========================================================================

##===========================================================================
##                     Initialisation after system reset
##===========================================================================
	.text
	.global __start
	.extern __init_board

__start:
#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
	bl __init_cpu			## Initialise the CPU
#endif

#if defined(CYG_HAL_STARTUP_STUBS)
        lwi r3, 0x2082000f              ## Single steps are implemented using hardware interrupts,
        mtder r3                        ## breakpoint use program exceptions
#elif defined(CYG_HAL_STARTUP_ROM)
        lwi r3, 0x00000000		## Never enter debug mode for ROM startup, leave everything
        mtder r3			## as it is for RAM startup
#endif

	## Setup the global offset table
        lwi r2, _GLOBAL_OFFSET_TABLE_

	## Setup time base register to zero
	xor r3, r3, r3
	mtspr TBL_W, r3
	xor r4, r4, r4
	mtspr TBU_W, r4
	
	## Setup the stack
#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
	lwi sp, __interrupt_stack
	mtspr sprg0, sp			## Save in sprg0 for later use
#endif

	## Call platform specific hardware initialization.
	## It is not safe to rely on memory configuration until after this point.
#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
	bl __init_hardware
#else
        bl __init_vectors
#endif

#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
        ## Initialise the vsr table
	.extern __ram_exc_vsr_data_start
	.extern __rom_exc_vsr_data_start
	.extern __ram_exc_vsr_data_end
	lwi r3, __rom_exc_vsr_data_start	## r3 = rom start
	lwi r4, __ram_exc_vsr_data_start	## r4 = ram start
	lwi r5, __ram_exc_vsr_data_end		## r5 = ram end

	cmplw r4, r5				## skip if no data
	beq 2f
1:
	lwz r0, 0(r3)				## get word from rom
	stw r0, 0(r4)				## copy it to ram
	addi r3, r3, 4				## increment read pointer
	addi r4, r4, 4				## increment write pointer
	cmplw r4, r5				## loop if not yet done
	blt 1b
2:


        ## Initialise the isr table
	.extern __ram_isr_handlers_data_start
	.extern __rom_isr_handlers_data_start
	.extern __ram_isr_handlers_data_end
	lwi r3, __rom_isr_handlers_data_start	## r3 = rom start
	lwi r4, __ram_isr_handlers_data_start	## r4 = ram start
	lwi r5, __ram_isr_handlers_data_end	## r5 = ram end

	cmplw r4, r5				## skip if no data
	beq 2f
1:
	lwz r0, 0(r3)				## get word from rom
	stw r0, 0(r4)				## copy it to ram
	addi r3, r3, 4				## increment read pointer
	addi r4, r4, 4				## increment write pointer
	cmplw r4, r5				## loop if not yet done
	blt 1b
2:


        ## Initialise the isr data
	.extern __ram_isr_data_data_start
	.extern __rom_isr_data_data_start
	.extern __ram_isr_data_data_end
	lwi r3, __rom_isr_data_data_start	## r3 = rom start
	lwi r4, __ram_isr_data_data_start	## r4 = ram start
	lwi r5, __ram_isr_data_data_end		## r5 = ram end

	cmplw r4, r5				## skip if no data
	beq 2f
1:
	lwz r0, 0(r3)				## get word from rom
	stw r0, 0(r4)				## copy it to ram
	addi r3, r3, 4				## increment read pointer
	addi r4, r4, 4				## increment write pointer
	cmplw r4, r5				## loop if not yet done
	blt 1b
2:


        ## Initialise the isr obj
	.extern __ram_isr_obj_data_start
	.extern __rom_isr_obj_data_start
	.extern __ram_isr_obj_data_end
	lwi r3, __rom_isr_obj_data_start	## r3 = rom start
	lwi r4, __ram_isr_obj_data_start	## r4 = ram start
	lwi r5, __ram_isr_obj_data_end		## r5 = ram end

	cmplw r4, r5				## skip if no data
	beq 2f
1:
	lwz r0, 0(r3)				## get word from rom
	stw r0, 0(r4)				## copy it to ram
	addi r3, r3, 4				## increment read pointer
	addi r4, r4, 4				## increment write pointer
	cmplw r4, r5				## loop if not yet done
	blt 1b
2:


	## Initialise memory
	lwi r3, __rom_data_start	## r3 = rom start
	lwi r4, __ram_data_start	## r4 = ram start
	lwi r5, __ram_data_end		## r5 = ram end

	cmplw r4, r5			## skip if no data
	beq 2f
1:
	lwz r0, 0(r3)			## get word from rom
	stw r0, 0(r4)			## copy it to ram
	addi r3, r3, 4			## increment read pointer
	addi r4, r4, 4			## increment write pointer
	cmplw r4, r5			## loop if not yet done
	blt 1b
2:

#endif

	## Clear BSS
	.extern __bss_start
	.extern __bss_end
	lwi r3, __bss_start		## r3 = start
	lwi r4, __bss_end		## r4 = end
	li r0, 0			## r0 = 0

	cmplw r3, r4			## skip if no bss
	beq 2f
1:
	stw r0, 0(r3)			## store zero
	addi r3, r3, 4			## increment write pointer
	cmplw r3, r4			## compare
	blt 1b				## loop if not yet done
2:

	# Clear SBSS
	.extern __sbss_start
	.extern __sbss_end
	lwi r3, __sbss_start		## r3 = start
	lwi r4, __sbss_end		## r4 = end

	cmplw r3, r4			## skip if no sbss
	beq 2f
1:
	stw r0, 0(r3)			## store zero
	addi r3, r3, 4			## increment write pointer
	cmplw r3, r4			## compare
	blt 1b				## loop if not yet done
2:

	##---------------------------
 	## Memory access is safe now.
	## We can start calling c-functions that rely on unitialized data

	## Set up stack for calls to C-code
	subi sp, sp, 12				## Make space
	li r0, 0
	stw r0, 0(sp)				## clear back chain
	stw r0, 8(sp)				## zero return pc
	stwu sp, -ppc_stack_frame_size(sp)	## new stack frame

#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
	## Set up stack for calls to C-code
	subi sp, sp, 12				## Make space
	li r0, 0
	stw r0, 0(sp)				## clear back chain
	stw r0, 8(sp)				## zero return pc
	stwu sp, -ppc_stack_frame_size(sp)	## new stack frame

	## Initialize MMU
	.extern hal_MMU_init
	bl hal_MMU_init

	## Enable MMU so we can safely enable caches
	lwi r3, CYG_MSR
	sync
	mtmsr r3
	sync

	## Enable caches
	.extern hal_enable_caches
	bl hal_enable_caches
#endif

	## Call C++ constructors
	.extern cyg_hal_invoke_constructors
	bl cyg_hal_invoke_constructors

#ifdef CYGDBG_HAL_DEBUG_GDB_INCLUDE_STUBS
	## Initialize the GDB stub
	.extern initialize_stub
	bl initialize_stub
#endif

	##--------------------------------
	## And finally start up the system
	.extern cyg_start
	bl cyg_start
9:					## if we return, loop
	b 9b



	##------------------------------------------------------------------------
	## Code for putting the CPU in a post-reset state disabling MMU and caches
	##
	##
__init_cpu:
	lwi r3, (CYG_MSR & ~(MSR_IR | MSR_DR))
	sync
	mtmsr r3
	sync
	blr

	##-------------------------------------------------------------------------
	## Code for initialising the hardware
	##
	##
#if defined(CYG_HAL_STARTUP_ROM) || defined(CYG_HAL_STARTUP_STUBS)
__init_hardware:
	lis r0, -16	## Just do some stuff to get the processor
	mtspr 638, r0	## operational

	li r0, 0
	mtspr 560, r0

	mflr r31	## Save the return address, cannot rely on the stack yet
	bl __init_board	## and this might be a C-function
	mtlr r31	## restore the return address and

	blr		## return
#else
__init_vectors:
	mflr r31        ## Save the return address, cannot rely on the stack yet
        bl __init_isr   ## and this might be a C-function
	mtlr r31        ## restore the return address and

	blr		## return
#endif
