#ifndef TOUCAN_ISRSOURCE_CXX
#define TOUCAN_ISRSOURCE_CXX

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include "cyg/hal/toucan_isrsource.h"

#if !defined(CYG_HAL_STARTUP_STUBS) && defined(CYG_HAL_MPC555)
#include "cyg/hal/imb3_isrdispatch.h"
#include "cyg/hal/mpc555_regs.h"
#include "cyg/hal/hal_intr.h"
#include "cyg/hal/hal_io.h"

//---------------------
// Class implementation
//---------------------
// Public constructor(s) and destructor
TOUCAN_isrSource::TOUCAN_isrSource(IMB3_isrDispatcher * a_dispatcher,
                                   unsigned char a_isrpriority,
                                   TOUCAN_isrSource::TOUCAN_IDENTIFICATION a_identification)
    : IMB3_isrSource(a_dispatcher),
      m_priority(a_isrpriority),
      m_identification(a_identification)
{
  CYG_ASSERT(a_isrpriority < 32, "Interrupt priorities 0..31");

  a_dispatcher->attachToPriority(this, a_isrpriority);

  // Now program the priority into the hardware
  cyg_uint16 irl  = a_isrpriority % 8;
  cyg_uint16 ilbs = a_isrpriority / 8; 
  cyg_uint16 canicr;

  switch(a_identification)
  {
    case TOUCAN_B:
      HAL_READ_UINT16(CYGARC_REG_IMM_CANICR_B, canicr);
      break;
    case TOUCAN_A:
    default:
      HAL_READ_UINT16(CYGARC_REG_IMM_CANICR_A, canicr);
      break;
  }

  irl = irl << CYGARC_REG_IMM_CANICR_IRL_SHIFT;
  irl &= (cyg_uint16)CYGARC_REG_IMM_CANICR_IRL;
  ilbs = ilbs << CYGARC_REG_IMM_CANICR_ILBS_SHIFT;
  ilbs &= (cyg_uint16)CYGARC_REG_IMM_CANICR_ILBS;

  canicr &= ~((cyg_uint16)CYGARC_REG_IMM_CANICR_IRL);
  canicr &= ~((cyg_uint16)CYGARC_REG_IMM_CANICR_ILBS);

  canicr |= irl;
  canicr |= ilbs;

  switch(a_identification)
  {
    case TOUCAN_B:
      HAL_WRITE_UINT16(CYGARC_REG_IMM_CANICR_B, canicr);
      break;
    case TOUCAN_A:
    default:
      HAL_WRITE_UINT16(CYGARC_REG_IMM_CANICR_A, canicr);
      break;
  }
}

// Provide an (empty) virtual destructor
TOUCAN_isrSource::~TOUCAN_isrSource()
{
}

//------------------------
// Public member functions
//------------------------
// findInterruptingVector
cyg_uint32 TOUCAN_isrSource::findInterruptingVector(unsigned char a_priority)
{
  CYG_UNUSED_PARAM(unsigned char, a_priority);
  cyg_uint16 imask;
  cyg_uint16 iflag;
  cyg_uint16 canctrl0_canctrl1;
  cyg_uint16 tcnmcr;
  cyg_uint16 estat;

  switch(m_identification)
  {
    case TOUCAN_B:
      HAL_READ_UINT16(CYGARC_REG_IMM_IMASK_B, imask);
      HAL_READ_UINT16(CYGARC_REG_IMM_IFLAG_B, iflag);
      HAL_READ_UINT16(CYGARC_REG_IMM_CANCTRL0_B_CANCTRL1_B, canctrl0_canctrl1);
      HAL_READ_UINT16(CYGARC_REG_IMM_TCNMCR_B, tcnmcr);
      HAL_READ_UINT16(CYGARC_REG_IMM_ESTAT_B, estat);
      break;
    case TOUCAN_A:
    default:
      HAL_READ_UINT16(CYGARC_REG_IMM_IMASK_A, imask);
      HAL_READ_UINT16(CYGARC_REG_IMM_IFLAG_A, iflag);
      HAL_READ_UINT16(CYGARC_REG_IMM_CANCTRL0_A_CANCTRL1_A, canctrl0_canctrl1);
      HAL_READ_UINT16(CYGARC_REG_IMM_TCNMCR_A, tcnmcr);
      HAL_READ_UINT16(CYGARC_REG_IMM_ESTAT_A, estat);
      break;
  }

  iflag &= imask; // Do not respond to masked interrupts
  
  if(iflag && (m_identification == TOUCAN_A))
  { // Don't poll the iflag register if the interrupt was caused by the error register
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B0;

    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B1;
 
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B2;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B3;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B4;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B5;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B6;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B7;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B8;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B9;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B10;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B11;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B12;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B13;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B14;
 
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_B15;
  }

  if(iflag && (m_identification == TOUCAN_B))
  { // Don't poll the iflag register if the interrupt was caused by the error register
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B0;

    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B1;
 
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B2;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B3;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B4;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B5;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B6;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B7;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B8;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B9;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B10;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B11;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B12;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B13;
  
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B14;
 
    iflag = (iflag >> 1);
    if(iflag & 1)
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_B15;
  }

  if(m_identification == TOUCAN_A)
  { // Do not respond to masked interrupts
    if((estat & CYGARC_REG_IMM_ESTAT_BOFFINT) &&
       (canctrl0_canctrl1 & CYGARC_REG_IMM_CANCTRL0_CANCTRL1_BOFFMSK))
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_BOFF;

    if((estat & CYGARC_REG_IMM_ESTAT_ERRINT)  &&
       (canctrl0_canctrl1 & CYGARC_REG_IMM_CANCTRL0_CANCTRL1_ERRMSK))
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_ERR;

    if((estat & CYGARC_REG_IMM_ESTAT_WAKEINT) &&
       (tcnmcr & CYGARC_REG_IMM_TCNMCR_WAKEMSK))
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANA_WU;
  }

  if(m_identification == TOUCAN_B)
  { // Do not respond to masked interrupts
    if((estat & CYGARC_REG_IMM_ESTAT_BOFFINT) &&
       (canctrl0_canctrl1 & CYGARC_REG_IMM_CANCTRL0_CANCTRL1_BOFFMSK))
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_BOFF;

    if((estat & CYGARC_REG_IMM_ESTAT_ERRINT)  &&
       (canctrl0_canctrl1 & CYGARC_REG_IMM_CANCTRL0_CANCTRL1_ERRMSK))
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_ERR;

    if((estat & CYGARC_REG_IMM_ESTAT_WAKEINT) &&
       (tcnmcr & CYGARC_REG_IMM_TCNMCR_WAKEMSK))
      return CYGNUM_HAL_INTERRUPT_IMB3_TOUCANB_WU;
  }

  // Shouldn't come here. Give every possible control path
  // a return value.
  return 0;
}

// enableInterrupts
void TOUCAN_isrSource::enableInterrupts(void)
{
  // Nothing to be done. TOUCAN cannot be enabled or disabled. Prevent the class
  // from being abstract though.
}

// disableInterrupts
void TOUCAN_isrSource::disableInterrupts(void)
{
  // Nothing to be done. TOUCAN cannot be enabled or disabled. Prevent the class
  // from being abstract though.
}
#endif // if !defined ...

#endif // TOUCAN_ISRSOURCE_CXX
