#ifndef CMSVAR_INL
#define CMSVAR_INL

//----------------------------------
// Includes and forward declarations
//----------------------------------

//----------------------
// Class implementations
//----------------------
// Constructor
template <class Type>
ROBasicClientVar<Type>::ROBasicClientVar(unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_rxcobid,
                                         can_io_handle_t a_handle,
                                         bool a_numeric)
    : CMS_BasicClientVariable(CMS_Variable::Read_Only, 
                              a_inhibittime, 
                              a_priority, 
                              a_name, 
                              sizeof(Type), 
                              0, a_rxcobid, 
                              a_handle,
                              a_numeric)
{
}

// readVariable
template <class Type>
CMS_Error ROBasicClientVar<Type>::readVariable(Type * a_data, 
                                               unsigned char a_response[8], 
                                               unsigned int a_timeout)
{
  CMS_Error result = read(a_response, 0, a_timeout);
  if(result == CMS_NOERR)
    memcpy(a_data, a_response, sizeof(Type));

  return result;
}

// Constructor
template <class Type>
RWBasicClientVar<Type>::RWBasicClientVar(unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                         can_io_handle_t a_handle,
                                         bool a_numeric)
    : CMS_BasicClientVariable(CMS_Variable::Read_Write, 
                              a_inhibittime, 
                              a_priority, 
                              a_name, 
                              sizeof(Type), 
                              a_txcobid, a_rxcobid,
                              a_handle,
                              a_numeric)
{
}

// readVariable
template <class Type>
CMS_Error RWBasicClientVar<Type>::readVariable(Type * a_data, 
                                               unsigned char a_response[8], 
                                               unsigned int a_timeout)
{
  CMS_Error result = read(a_response, 0, a_timeout);
  if(result == CMS_NOERR)
    memcpy(a_data, a_response, sizeof(Type));

  return result;
}

// writeVariable
template <class Type>
CMS_Error RWBasicClientVar<Type>::writeVariable(Type a_data, 
                                                unsigned char a_response[8], 
                                                unsigned int a_timeout)
{
  memcpy(a_response, &a_data, sizeof(Type));
  return write(a_response, 0, a_timeout);
}

// Constructor
template <class Type>
WOBasicClientVar<Type>::WOBasicClientVar(unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_txcobid,
                                         can_io_handle_t a_handle,
                                         bool a_numeric)
    : CMS_BasicClientVariable(CMS_Variable::Write_Only, 
                              a_inhibittime, 
                              a_priority, 
                              a_name, 
                              sizeof(Type), 
                              a_txcobid, 0,
                              a_handle,
                              a_numeric)
{
}

// writeVariable
template <class Type>
CMS_Error WOBasicClientVar<Type>::writeVariable(Type a_data, 
                                                unsigned char a_response[8], 
                                                unsigned int a_timeout)
{
  memcpy(a_response, &a_data, sizeof(Type));
  return write(a_response, 0, a_timeout);
}

// Constructor
template <class Type>
ROBasicServerVar<Type>::ROBasicServerVar(unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_txcobid,
                                         can_io_handle_t a_handle,
                                         bool a_numeric)
    : CMS_BasicServerVariable(CMS_Variable::Read_Only, 
                              a_inhibittime, 
                              a_priority, 
                              a_name, 
                              sizeof(Type), 
                              a_txcobid, 0,
                              a_handle,
                              a_numeric)
{
}

// updateVariable
template <class Type>
void ROBasicServerVar<Type>::updateVariable(Type a_data)
{
  update((unsigned char *)&a_data);
}

// readResponse
template <class Type>
bool ROBasicServerVar<Type>::readResponse(unsigned char * /*a_buffer*/, 
                                          unsigned char /*a_mux*/)
{ // This should never be called for this type of variable. Might as wel make the class
  // no longer abstract
  return true;
}

// writeResponse
template <class Type>
bool ROBasicServerVar<Type>::writeResponse(unsigned char * /*a_buffer*/, 
                                           unsigned char /*a_mux*/)
{ // This should never be called for this type of variable. Might as wel make the class
  // no longer abstract
  return true;
}

// Constructor
template <class Type>
RWBasicServerVar<Type>::RWBasicServerVar(unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                         can_io_handle_t a_handle,
                                         bool a_numeric)
    : CMS_BasicServerVariable(CMS_Variable::Read_Write, 
                              a_inhibittime, 
                              a_priority, 
                              a_name, 
                              sizeof(Type), 
                              a_txcobid, a_rxcobid, 
                              a_handle,
                              a_numeric)
{
}

// readResponse
template <class Type>
bool RWBasicServerVar<Type>::readResponse(unsigned char * a_buffer, 
                                          unsigned char /*a_mux*/)
{
  Type tmp;

  if(dataRequest(&tmp, a_buffer))
  {
    encode(a_buffer, &tmp, m_length);
    return true;
  }

  // if no data available, application response field is filled by dataRequest
  return false;
}

// writeResponse
template <class Type>
bool RWBasicServerVar<Type>::writeResponse(unsigned char * a_buffer, 
                                           unsigned char /*a_mux*/)
{
  Type tmp;
  decode(&tmp, a_buffer, m_length);

  if(dataIndication(tmp, a_buffer))
  {
    encode(a_buffer, &tmp, m_length); // On success, the same value should be returned.
    return true;                      // Make sure the application did not mess with a_buffer
  }

  return false;
}

// Constructor
template <class Type>
WOBasicServerVar<Type>::WOBasicServerVar(unsigned int a_inhibittime,
                                         unsigned char a_priority,
                                         const char * a_name,
                                         CAN_msgid a_rxcobid,
                                         can_io_handle_t a_handle,
                                         bool a_numeric)
    : CMS_BasicServerVariable(CMS_Variable::Write_Only, 
                              a_inhibittime, 
                              a_priority, 
                              a_name, 
                              sizeof(Type), 
                              0, a_rxcobid, 
                              a_handle,
                              a_numeric)
{
}

// readResponse
template <class Type>
bool WOBasicServerVar<Type>::readResponse(unsigned char * /*a_buffer*/, 
                                          unsigned char /*a_mux*/)
{ // This should never be called for this type of variable. Might as wel make the class
  // no longer abstract
  return true;
}

// writeResponse
template <class Type>
bool WOBasicServerVar<Type>::writeResponse(unsigned char * a_buffer, 
                                           unsigned char /*a_mux*/)
{
  Type tmp;
  decode(&tmp, a_buffer, m_length);
  dataIndication(tmp);

  return true;
}

// Constructor
template<class Type>
ROMuxClientVar<Type>::ROMuxClientVar(unsigned int a_inhibittime,
                                     unsigned char a_priority,
                                     const char * a_name,
                                     CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                     can_io_handle_t a_handle,
                                     bool a_numeric)
    : CMS_MultiplexedClientVariable(CMS_Variable::Read_Only, 
                                    a_inhibittime, a_priority, 
                                    a_name, sizeof(Type), 
                                    a_txcobid, a_rxcobid, 
                                    a_handle, 
                                    a_numeric)
{
}

// readVariable
template<class Type>
CMS_Error ROMuxClientVar<Type>::readVariable(Type * a_data, 
                                             unsigned char a_mux, 
                                             unsigned char a_response[8], 
                                             unsigned int a_timeout)
{
  CMS_Error result = read(a_response, a_mux, a_timeout);
  if(result == CMS_NOERR)
    memcpy(a_data, a_response, sizeof(Type));

  return result;
}

// Constructor
template<class Type>
ROMuxServerVar<Type>::ROMuxServerVar(unsigned int a_inhibittime,
                                     unsigned char a_priority,
                                     const char * a_name,
                                     CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                     can_io_handle_t a_handle,
                                     bool a_numeric)
    : CMS_MultiplexedServerVariable(CMS_Variable::Read_Only,
                                    a_inhibittime, a_priority,
                                    a_name, sizeof(Type),
                                    a_txcobid, a_rxcobid,
                                    a_handle,
                                    a_numeric)
{
}

// writeResponse
template <class Type>
bool ROMuxServerVar<Type>::writeResponse(unsigned char * /*a_buffer*/, 
                                         unsigned char /*a_mux*/)
{ // This should never be called for this type of variable. Might as wel make the class
  // no longer abstract
  return true;
}

// readResponse
template <class Type>
bool ROMuxServerVar<Type>::readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux)
{
  Type tmp;

  if(dataRequest(&tmp, a_mux, a_buffer))
  {
    encode(a_buffer, &tmp, m_length);
    return true;
  }

  // if no data available, application response field is filled by dataRequest
  return false;
}

// Constructor
template <class Type>
WOMuxClientVar<Type>::WOMuxClientVar(unsigned int a_inhibittime,
                                     unsigned char a_priority,
                                     const char * a_name,
                                     CAN_msgid a_cobid,
                                     can_io_handle_t a_handle,
                                     bool a_numeric)
    : CMS_MultiplexedClientVariable(CMS_Variable::Write_Only,
                                    a_inhibittime, a_priority, a_name,
                                    sizeof(Type),
                                    a_cobid, 0,
                                    a_handle, a_numeric)
{
}

// writeVariable
template <class Type>
CMS_Error WOMuxClientVar<Type>::writeVariable(Type a_data, 
                                              unsigned char a_mux, 
                                              unsigned char a_response[8], 
                                              unsigned int a_timeout)
{
  memcpy(a_response, &a_data, sizeof(Type));
  return write(a_response, a_mux, a_timeout);
}

// Constructor
template <class Type>
WOMuxServerVar<Type>::WOMuxServerVar(unsigned int a_inhibittime,
                                     unsigned char a_priority,
                                     const char * a_name,
                                     CAN_msgid a_cobid,
                                     can_io_handle_t a_handle,
                                     bool a_numeric)
    : CMS_MultiplexedServerVariable(CMS_Variable::Write_Only,
                                    a_inhibittime, a_priority, a_name,
                                    sizeof(Type),
                                    0, a_cobid,
                                    a_handle, a_numeric)
{
}

// readResponse
template <class Type>
bool WOMuxServerVar<Type>::readResponse(unsigned char * /*a_buffer*/, 
                                        unsigned char /*a_mux*/)
{ // This should never be called for this type of variable. Might as wel make the class
  // no longer abstract
  return true;
}

// writeResponse
template <class Type>
bool WOMuxServerVar<Type>::writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux)
{
  Type tmp;
  decode(&tmp, a_buffer, m_length);
  dataIndication(tmp, a_mux);

  return true;
}

// Constructor
template <class Type>
RWMuxClientVar<Type>::RWMuxClientVar(unsigned int a_inhibittime, 
                                     unsigned char a_priority,
                                     const char * a_name,
                                     CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                     can_io_handle_t a_handle,
                                     bool a_numeric)
    : CMS_MultiplexedClientVariable(CMS_Variable::Read_Write,
                                    a_inhibittime, a_priority, a_name,
                                    sizeof(Type),
                                    a_txcobid, a_rxcobid,
                                    a_handle, a_numeric)
{
}

// readVariable
template <class Type>
CMS_Error RWMuxClientVar<Type>::readVariable(Type * a_data, 
                                             unsigned char a_mux, 
                                             unsigned char a_response[8], 
                                             unsigned int a_timeout)
{
  CMS_Error result = read(a_response, a_mux, a_timeout);
  if(result == CMS_NOERR)
    memcpy(a_data, a_response, sizeof(Type));

  return result;
}

// writeVariable
template <class Type>
CMS_Error RWMuxClientVar<Type>::writeVariable(Type a_data,
                                              unsigned char a_mux,
                                              unsigned char a_response[8],
                                              unsigned int a_timeout)
{
  memcpy(a_response, &a_data, sizeof(Type));
  return write(a_response, a_mux, a_timeout);
}

// Constructor
template <class Type>
RWMuxServerVar<Type>::RWMuxServerVar(unsigned int a_inhibittime,
                                     unsigned char a_priority,
                                     const char * a_name,
                                     CAN_msgid a_txcobid, CAN_msgid a_rxcobid,
                                     can_io_handle_t a_handle,
                                     bool a_numeric)
    : CMS_MultiplexedServerVariable(CMS_Variable::Read_Write,
                                    a_inhibittime, a_priority, a_name,
                                    sizeof(Type),
                                    a_txcobid, a_rxcobid,
                                    a_handle, a_numeric)
{
}

// readResponse
template <class Type>
bool RWMuxServerVar<Type>::readResponse(unsigned char * a_buffer, 
                                        unsigned char a_mux)
{
  Type tmp;

  if(dataRequest(&tmp, a_mux, a_buffer))
  {
    encode(a_buffer, &tmp, m_length);
    return true;
  }

  // if no data available, application response field is filled by dataRequest
  return false;
}

// writeResponse
template <class Type>
bool RWMuxServerVar<Type>::writeResponse(unsigned char * a_buffer, 
                                         unsigned char a_mux)
{
  Type tmp;
  decode(&tmp, a_buffer, m_length);

  if(dataIndication(tmp, a_mux, a_buffer))
  {
    encode(a_buffer, &tmp, m_length); // On success, the same value should be returned.
    return true;                      // Make sure the application did not mess with a_buffer
  }

  return false;
}

#endif // CMSVAR_INL
