#ifndef COMOBJ_H
#define COMOBJ_H

//----------------------------------
// Includes and forward declarations
//----------------------------------
#include <cyg/io/caltypes.h>
#include <cyg/io/io.h>
#include <cyg/kal/ksync.h>
class CommunicationObject;
class CanMessageDispatcher;

//-------------------
// Class declarations
//-------------------
// Class CommunicationObjectClient
class CommunicationObjectClient
{
  public:
                                CommunicationObjectClient() {}
    virtual                     ~CommunicationObjectClient() {}

    virtual void                dataIndication(unsigned char a_data[8],
                                               unsigned char a_length,
                                               CommunicationObject * a_cob) = 0;
};

// Class CommunicationObject
class CommunicationObject
{
  friend class CanMessageDispatcher;
  public:
    typedef enum {
      tx, rx, RTR_client, RTR_server } COB_Class;

    COB_Class                   getCOBClass(void)
                                {
                                return m_class;
                                }
    CAN_msgid                   getCOBid(void)
                                {
                                return m_cobid;
                                }

    virtual                     ~CommunicationObject();
    virtual bool                create(void) = 0;
    virtual void                destroy(void) = 0;

  private:
    void                        dataIndication(unsigned char * a_data, unsigned char a_length);

  protected:
                                // Prevent the base class from being instantiated directly
                                CommunicationObject(CAN_msgid a_cobid, 
                                                    can_io_handle_t a_handle,
                                                    CommunicationObjectClient * a_client,
                                                    unsigned int a_inhibittime = 0);
    void                        waitForInhibit(void);
    
    COB_Class                   m_class;
    can_io_handle_t             m_handle;
    CommunicationObjectClient * m_client;
    CAN_msgid                   m_cobid;
    CanMessageDispatcher *      m_dispatcher;
    static KCondition *         m_tickcondition;
    static unsigned long        m_tickcount;

  private:
    unsigned long               m_marker;
    unsigned int                m_inhibitticks;
};

// Class TxCOB
class TxCOB : public CommunicationObject
{
  public:
                                TxCOB(CAN_msgid a_cob_id,
                                      can_io_handle_t a_handle,
                                      CommunicationObjectClient * a_client,
                                      unsigned int a_inhibittime = 0);
    virtual                     ~TxCOB();

    virtual bool                create(void);
    virtual void                destroy(void);

    bool                        transmit(const unsigned char * a_data, unsigned char a_length);
};

// Class RxCOB
class RxCOB : public CommunicationObject
{
  public:
                                RxCOB(CAN_msgid a_cob_id,
                                      can_io_handle_t a_handle,
                                      CommunicationObjectClient * a_client);
    virtual                     ~RxCOB();

    virtual bool                create(void);
    virtual void                destroy(void);
};

// Class RTR_Tx_COB
class RTR_ClientCOB : public CommunicationObject
{
  public:
                                RTR_ClientCOB(CAN_msgid a_cob_id,
                                              can_io_handle_t a_handle,
                                              CommunicationObjectClient * a_client,
                                              unsigned int a_inhibittime = 0);
    virtual                     ~RTR_ClientCOB();

    virtual bool                create(void);
    virtual void                destroy(void);

    bool                        request(unsigned char a_length);
};

// Class RTR_Rx_COB
class RTR_ServerCOB : public CommunicationObject
{
  public:
                                RTR_ServerCOB(CAN_msgid a_cob_id,
                                              can_io_handle_t a_handle,
                                              CommunicationObjectClient * a_client,
                                              unsigned int a_inhibittime = 0);
    virtual                     ~RTR_ServerCOB();

    virtual bool                create(void);
    virtual void                destroy(void);

    bool                        update(const unsigned char * a_data, unsigned char a_length);
    bool                        transmit(const unsigned char * a_data, unsigned char a_length);
};

#endif // COMOBJ_H
